/// <reference types="react" />
export {};
declare global {
    type NewMembers<T, U> = keyof T & keyof U;
    type PickRename<T, K extends keyof T, R extends PropertyKey> = Omit<T, K> & {
        [P in R]: T[K];
    };
    type KeyOf<T> = keyof T;
    type ArgumentTypes<F extends Function> = F extends (...args: infer A) => any ? A : never;
    type ArglessCallfn<T> = () => T;
    type Records<T> = {
        [K in keyof T]: keyof T[K];
    };
    type RecordsPartial<T> = Partial<Records<T>>;
    type HardRecords<T, O extends keyof any, C> = Record<keyof Records<Omit<T, O>>, C>;
    type HardRecordsPartial<T, O extends keyof any, C> = Partial<Record<keyof Records<Omit<T, O>>, C>>;
    type Nullable<T> = T | null;
    type Undefineable<T = any> = T | undefined;
    type PosibleUndefined<T> = T | null | undefined;
    type TypeName<T> = T extends string ? "string" : T extends number ? "number" : T extends boolean ? "boolean" : T extends undefined ? "undefined" : T extends Function ? "function" : "object";
    type AnyFunction = (...args: any[]) => any;
    type MutableRequired<T> = {
        -readonly [P in keyof T]-?: T[P];
    };
    type ReadonlyPartial<T> = {
        +readonly [P in keyof T]+?: T[P];
    };
    type Unpacked<T> = T extends (infer U)[] ? U : T extends (...args: any[]) => infer U ? U : T extends Promise<infer U> ? U : T;
    type StringOrNumber = string | number;
    type StringOrBoolean = string | boolean;
    type BooleanOrNumber = boolean | number;
    type StringOrSymbol = string | symbol;
    type StringBooleanNumber = string | boolean | number;
    type StringBooleanNumberFunction = string | boolean | number | Function;
    type StringBooleanNumberSymbol = string | boolean | number | symbol;
    type Callback<T> = (...args: any[]) => T;
    /**
     * React tools
     */
    const Geact: Geact;
    namespace Geact {
        type SetState<T> = ((newValue: T) => void) | ((prevValue: T) => T);
        type UseStateArg<T> = T | (() => T);
    }
    interface Geact {
        /**
         * New React DOM render method. Requires React 18+
         * @param component
         * @param element
         */
        render(component: React.ReactNode, element: string): void;
        /**
         * React DOM render method to render the DOM and root element automatically. Requires React 18+ and an component with call syntax.
         * @param InitComponent {ComponentClass} Uses the given component to render the DOM and the required HTML root element
         */
        renderAuto<P = {}>(InitComponent: React.ElementType, props?: P): void;
        /**
         * React DOM legacy render method
         * @param component
         * @param element
         * @param callback
         */
        renderLegacy(component: React.DOMElement<React.DOMAttributes<Element>, Element>, element: string, callback?: Callback<void>): void;
        /**
         * Custom React state hook
         */
        useState: <T extends any>(defaultValue: Geact.UseStateArg<T>) => [T, Geact.SetState<T>];
        useForceRender(): any;
    }
    interface String {
        format: (tofor: Record<StringOrNumber, StringBooleanNumber>) => string;
        toBoolean(): boolean;
        node(): {
            custom: (code: number) => string;
            bright: string;
            dim: string;
            underscore: string;
            blink: string;
            reverse: string;
            hidden: string;
            fg: {
                black: string;
                red: string;
                green: string;
                yellow: string;
                blue: string;
                magenta: string;
                cyan: string;
                white: string;
            };
            bg: {
                black: string;
                red: string;
                green: string;
                yellow: string;
                blue: string;
                magenta: string;
                cyan: string;
                white: string;
            };
        };
    }
    interface Navigator {
        /**
         * @usage
         * ```ts
         * window.permission("clipboard-write").then((pm: PermissionStatus) => {
         *   console.log(pm.state)
         * });
         * ```
         * @param permission
         * @returns `"granted"`, `"denied"` or `"prompt"`
         */
        permission(permission: string): Promise<PermissionStatus>;
    }
    namespace Window {
        type EventHandlersMap = Records<GlobalEventHandlersEventMap>;
        type PreventerArgs = keyof EventHandlersMap | Array<keyof EventHandlersMap>;
    }
    interface Window {
        /**
         * Prevents event listener
         * @param prevent {Array<string>} `["contextmenu", "mousedown"]` or just a string
         */
        preventer(prevent: Window.PreventerArgs): void;
    }
    interface Location {
        /**
         * Retuns the given subpath from the current url
         * @param url Your url
         * @returns {string} Given subpath
         */
        getSubPath(url: string): string;
        parseURLParams(url: string): string[] | undefined;
        request<T = any>(url: string, callback: (status: XMLHttpRequest["status"], data: T) => void): void;
        /**
         * Sets an custom fake path to the current url
         * @param callback
         */
        setURL(callback: (set: (data?: any, unused?: string, url?: string | URL | null | undefined) => void, currentPath?: string | undefined) => void): void;
        /**
         * Checks if an string link is valid
         * @param input string
         * @returns {boolean} boolean
         */
        validURL(input: string): boolean;
    }
    namespace Console {
        type MessageSelf = Readonly<JSON>;
        type Message<T> = T | Callback<T>;
    }
    interface Console {
        styl(type: Omit<keyof typeof console, "Console">): (args: any) => void;
        /**
         * Usage of function callbacks in `console.log`
         */
        muc<T>(...data: Array<Console.Message<T>>): void;
    }
    interface JSON {
        /**
         * Checks if the given object is empty
         */
        isEmpty(obj: any): boolean;
        hasStructure(str: string): boolean;
        /**
         *
         * ### Remove elements from an object
         *
         * ----
         *
         * **Usage**
         * ```ts
         * const test = {
         *   firstName: "John",
         *   lastName: "Smith",
         *   id: 123,
         *   isEnabled: true,
         * };
         *
         * // Throws an error
         * const f = JSON.omit("firstName", test)?.firstName;
         * const fd = JSON.omit("firstName", test)?.lastName;
         *
         * console.log(JSON.stringify(fd));
         * ```
         * ----
         */
        omit<T = {}, R extends keyof any = "">(key: (keyof T | R) | (keyof T)[], obj: T): Undefineable<Omit<T, R>>;
    }
    interface Object {
        /**
         * **Usage**
         * ```ts
         * import { print, obj } from 'googlers-tools';
         *
         * const test = {
         *   firstName: "John",
         *   lastName: "Smith",
         *   id: 123,
         *   isEnabled: true,
         * };
         *
         * print.log(self =>
         *   self.stringify(
         *     obj.keys<boolean>(test)((key, index) => {
         *       return typeof test[key] == "boolean";
         *     }),
         *     null,
         *     4
         *   )
         * );
         * ```
         *
         * ----
         *
         * **Result**
         *
         * ```json
         * [
         *     false,
         *     false,
         *     false,
         *     true
         * ]
         * ```
         *
         */
        keysMap<R extends any = string, T extends object | {} = {}>(ojb: T): (callbackfn: (key: string, index: number) => R) => R[];
    }
    namespace Document {
        type FindCallback<T> = (element: T) => void;
        type FindRefObject<T> = React.MutableRefObject<T | undefined> | React.RefObject<T>;
    }
    interface Document {
        /**
         * **Usage**
         * ```ts
         * import { doc } from "googlers-tools";
         *
         * const htm = doc.findId("container")?.innerHTML
         * console.log(htm)
         *
         * // Won't work!
         * doc.findId("from")?.style.display = "dfsdf";
         *
         * // Works!
         * doc.findId<HTMLDivElement>("from", e => {
         *  e.style.display = "none"
         * });
         *
         * ```
         * ----
         * @param id
         * @param callback
         * @returns
         */
        findId<T = HTMLElement>(id: string, callback?: Document.FindCallback<T>): void | T | null;
        /**
         * **Usage**
         * ```ts
         * import { doc } from "googlers-tools";
         *
         * const htm = doc.findRef("container")?.innerHTML
         * console.log(htm)
         *
         * // Won't work!
         * doc.findRef("from")?.style.display = "dfsdf";
         *
         * // Works!
         * doc.findRef<HTMLDivElement>("from", e => {
         *  e.style.display = "none"
         * });
         *
         * ```
         * ----
         * @param ref
         * @param callback
         * @returns
         */
        findRef<T = HTMLElement>(ref: Document.FindRefObject<T>, callback?: Document.FindCallback<T>): void | T | undefined;
    }
}
