/**
 * Typing for Util/util
 */
declare namespace Util {
    type Types = Readonly<{
        /**
         * Used as an class decorator
         */
        ImplementsStatics: <T>() => <U extends T>(constructor: U) => void;
        returnUndefined: (value: undefined | any, theReturn: string | boolean | number) => any;
        stringToBoolean: (value: string) => boolean;
        /**
         * An simple function to check if it's not equals null, "null" or something similar
         * @param toCheck The to checkin value function return
         * @param defValue The return value if there was an equal with null, "null" or something similar
         * @returns {R}
         */
        typeCheck: <R extends any = any>(toCheck: R, defValue: R) => R;
        /**
         * Marks an class method as deprecated
         */
        deprecated: (deprecationReason: string) => any;
        isBoolean: (arg: unknown) => arg is boolean;
        isNull: (arg: unknown) => arg is null;
        isNullOrUndefined: (arg: unknown) => arg is null | undefined;
        isNumber: (arg: unknown) => arg is number;
        isString: (arg: unknown) => arg is string;
        isSymbol: (arg: unknown) => arg is symbol;
        isUndefined: (arg: unknown) => arg is undefined;
        isObject: (arg: unknown) => arg is NonNullable<object>;
        isFunction: (arg: unknown) => arg is Function;
        isPrimitive: (arg: unknown) => arg is boolean | null | number | string | symbol | undefined;
    }>;
}
declare type util = typeof util[keyof typeof util];
declare const util: Util.Types;
export { Util, util };
